local _

-- PartyFramesImproved.lua
-- Significant Portions of LUA are adapted from PartyFramesImproved by kiforsbe http://www.curse.com/addons/wow/unitframesimproved

-- Debug function. Adds message to the chatbox (only visible to the local player)
function dout(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

-- Additional debug info can be found on http://www.wowwiki.com/Blizzard_DebugTools
-- /framestack [showhidden]
--		showhidden - if "true" then will also display information about hidden frames
-- /eventtrace [command]
-- 		start - enables event capturing to the EventTrace frame
--		stop - disables event capturing
--		number - captures the provided number of events and then stops
--		If no command is given the EventTrace frame visibility is toggled. The first time the frame is displayed, event tracing is automatically started.
-- /dump expression
--		expression can be any valid lua expression that results in a value. So variable names, function calls, frames or tables can all be dumped.


function tokenize(str)
	local tbl = {};
	for v in string.gmatch(str, "[^ ]+") do
		tinsert(tbl, v);
	end
	return tbl;
end

-- CREATE THE ADDON MAIN INSTANCE
local PartyFramesImproved = CreateFrame('Button', 'PartyFramesImproved');

-- EVENT LISTENER TO MAKE SURE WE ENABLE THE ADDON AT THE RIGHT TIME
function PartyFramesImproved:PLAYER_ENTERING_WORLD()
	EnablePartyFramesImproved();
	-- Set some default settings
	if (characterSettings == nil) then
		PartyFramesImproved_LoadDefaultSettings()
	end
end

-- Event listener to make sure we've loaded our settings and that we apply them
function PartyFramesImproved:VARIABLES_LOADED()
	dout("PartyFramesImproved settings loaded!");
	-- Reset default settings if needed
		if (characterSettings == nil) then
			PartyFramesImproved_LoadDefaultSettings()
		end
		if (characterSettings["partyscale"] == nil) then
			StaticPopup_Show("PFI_LAYOUT_RESETDEFAULT");
			PartyFramesImproved_LoadDefaultSettings()
		end
		if (characterSettings["PartyMemberFrame1Y"] == nil) then
			StaticPopup_Show("PFI_LAYOUT_RESETDEFAULT");
			PartyFramesImproved_LoadDefaultSettings()
		end
	-- Load Player settings if saved settings check out
		PartyFramesImproved_ApplySettings(characterSettings);
end

-- Function to apply saved character settings
function PartyFramesImproved_ApplySettings(characterSettings)
	PartyFramesImproved_SetPartyFrameScale(characterSettings["partyscale"])
	PartyFramesImproved_SetPartyFrameY(characterSettings["PartyMemberFrame1Y"])
end

-- Function to set default settings
function PartyFramesImproved_LoadDefaultSettings()
	characterSettings = {}
	characterSettings["partyscale"] = "1.8"
	characterSettings["PartyMemberFrame1Y"] = "-20"
end

function PartyFramesImproved_ResetSettings()
	PartyFramesImproved_SetPartyFrameScale(1.8)
	PartyFramesImproved_SetPartyFrameY(-20)
end

-- HOOKING GAME FUNCTIONS AND SETTING THEM TO FIRE ADDON FUNCTION
function EnablePartyFramesImproved()
	hooksecurefunc("PartyMemberFrame_ToPlayerArt", PartyFramesImproved_PartyMemberFrame_ToPlayerArt); 
	hooksecurefunc("PartyMemberFrame_ToVehicleArt", PartyFramesImproved_PartyMemberFrame_ToVehicleArt);
	--hooksecurefunc("PartyMemberFrame_OnEvent", PartyFramesImproved_UpdateRange);
	--hooksecurefunc("PartyMemberFrame_UpdateLeader", PartyFramesImproved_UpdateRange);
	--hooksecurefunc("PartyMemberFrame_UpdateMember", PartyFramesImproved_UpdateRange);
	--hooksecurefunc("PartyMemberFrame_UpdateNotPresentIcon", PartyFramesImproved_PartyMemberFrame_UpdateNotPresentIcon);
	hooksecurefunc("HealthBar_OnValueChanged", PartyFramesImproved_Style_PartyMemberColor);
	hooksecurefunc("UnitFrameHealthBar_Update",PartyFramesImproved_Style_PartyMemberColor);
end

-- COLOR FUNTION TO CALL COLOR CODE WHENEVER A PARTY MEMBERS HEALTHBAR UPDATES OR MIGHT OTHERWISE REVERT TO THE DEFAULT COLOR.
function PartyFramesImproved_Style_PartyMemberColor()
	for i = 1, GetNumGroupMembers() - 1 do
		if not InCombatLockdown() then
			PartyMemberFrame1HealthBar:SetStatusBarColor(UnitColor("party1"));
			PartyMemberFrame1HealthBar.lockColor = true
			PartyMemberFrame2HealthBar:SetStatusBarColor(UnitColor("party2"));
			PartyMemberFrame2HealthBar.lockColor = true
			PartyMemberFrame3HealthBar:SetStatusBarColor(UnitColor("party3"));
			PartyMemberFrame3HealthBar.lockColor = true
			PartyMemberFrame4HealthBar:SetStatusBarColor(UnitColor("party4"));
			PartyMemberFrame4HealthBar.lockColor = true
		end
	end
end

-- STYLE - PARTY MEMEBER FRAME STYLE CHANGES
function PartyFramesImproved_PartyMemberFrame_ToPlayerArt(self)
	if not InCombatLockdown() then
		-- Text
			PartyMemberFrame1Name:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame1Name:SetPoint("TOPLEFT", 50,-8)
			PartyMemberFrame1HealthBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame1ManaBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
		-- Border Texture
			PartyMemberFrame1Texture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PartyFrame")
			PartyMemberFrame1Texture:SetPoint("TOPLEFT", 0, 6)
		-- Border Flash
			PartyMemberFrame1Flash:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PARTYFRAME-FLASH");
			PartyMemberFrame1Flash:SetPoint("TOPLEFT", 0, 6)
		-- Health Bar
			PartyMemberFrame1HealthBar:SetPoint("TOPLEFT",47,-3)
			PartyMemberFrame1HealthBar:SetHeight(17)
		-- Mana Bar
			-- PartyMemberFrame1ManaBar:SetHeight(6)
		-- Background
			PartyMemberFrame1Background:SetPoint("TOPLEFT",46,-3)
			PartyMemberFrame1Background:SetHeight(24)
			PartyMemberFrame1Background:SetWidth(70)
		-- Debuff Frames
			-- PartyMemberFrame1Debuff1:SetPoint("TOPLEFT",135,-12)
			-- PartyMemberFrame1Debuff1:SetWidth(25)
			-- PartyMemberFrame1Debuff1:SetHeight(25)
			-- PartyMemberFrame1Debuff2:SetWidth(25)
			-- PartyMemberFrame1Debuff2:SetHeight(25)
			-- PartyMemberFrame1Debuff3:SetWidth(25)
			-- PartyMemberFrame1Debuff3:SetHeight(25)
			-- PartyMemberFrame1Debuff4:SetWidth(25)
			-- PartyMemberFrame1Debuff4:SetHeight(25)
			
			
		-- Text
			PartyMemberFrame2Name:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame2Name:SetPoint("TOPLEFT", 50,-8)
			PartyMemberFrame2HealthBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame2ManaBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
		-- Border Texture
			PartyMemberFrame2Texture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PartyFrame")
			PartyMemberFrame2Texture:SetPoint("TOPLEFT", 0, 6)
		-- Border Flash
			PartyMemberFrame2Flash:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PARTYFRAME-FLASH");
			PartyMemberFrame2Flash:SetPoint("TOPLEFT", 0, 6)
		-- Health Bar
			PartyMemberFrame2HealthBar:SetPoint("TOPLEFT",47,-3)
			PartyMemberFrame2HealthBar:SetHeight(17)
		-- Mana Bar
			-- PartyMemberFrame2ManaBar:SetHeight(6)
		-- Background
			PartyMemberFrame2Background:SetPoint("TOPLEFT",46,-3)
			PartyMemberFrame2Background:SetHeight(24)
			PartyMemberFrame2Background:SetWidth(70)
		-- Debuff Frames
			-- PartyMemberFrame2Debuff1:SetPoint("TOPLEFT",135,-12)
			-- PartyMemberFrame2Debuff1:SetWidth(25)
			-- PartyMemberFrame2Debuff1:SetHeight(25)
			-- PartyMemberFrame2Debuff2:SetWidth(25)
			-- PartyMemberFrame2Debuff2:SetHeight(25)
			-- PartyMemberFrame2Debuff3:SetWidth(25)
			-- PartyMemberFrame2Debuff3:SetHeight(25)
			-- PartyMemberFrame2Debuff4:SetWidth(25)
			-- PartyMemberFrame2Debuff4:SetHeight(25)
			
			
		-- Text
			PartyMemberFrame3Name:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame3Name:SetPoint("TOPLEFT", 50,-8)
			PartyMemberFrame3HealthBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame3ManaBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
		-- Border Texture
			PartyMemberFrame3Texture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PartyFrame")
			PartyMemberFrame3Texture:SetPoint("TOPLEFT", 0, 6)
		-- Border Flash
			PartyMemberFrame3Flash:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PARTYFRAME-FLASH");
			PartyMemberFrame3Flash:SetPoint("TOPLEFT", 0, 6)
		-- Health Bar
			PartyMemberFrame3HealthBar:SetPoint("TOPLEFT",47,-3)
			PartyMemberFrame3HealthBar:SetHeight(17)
		-- Mana Bar
			-- PartyMemberFrame3ManaBar:SetHeight(6)
		-- Background
			PartyMemberFrame3Background:SetPoint("TOPLEFT",46,-3)
			PartyMemberFrame3Background:SetHeight(24)
			PartyMemberFrame3Background:SetWidth(70)
		-- Debuff Frames
			-- PartyMemberFrame3Debuff1:SetPoint("TOPLEFT",135,-12)
			-- PartyMemberFrame3Debuff1:SetWidth(25)
			-- PartyMemberFrame3Debuff1:SetHeight(25)
			-- PartyMemberFrame3Debuff2:SetWidth(25)
			-- PartyMemberFrame3Debuff2:SetHeight(25)
			-- PartyMemberFrame3Debuff3:SetWidth(25)
			-- PartyMemberFrame3Debuff3:SetHeight(25)
			-- PartyMemberFrame3Debuff4:SetWidth(25)
			-- PartyMemberFrame3Debuff4:SetHeight(25)
			
			
			
		-- Text
			PartyMemberFrame4Name:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame4Name:SetPoint("TOPLEFT", 50,-8)
			PartyMemberFrame4HealthBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
			PartyMemberFrame4ManaBarText:SetFont("Fonts\\FRIZQT__.TTF",7, "");
		-- Border Texture
			PartyMemberFrame4Texture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PartyFrame")
			PartyMemberFrame4Texture:SetPoint("TOPLEFT", 0, 6)
		-- Border Flash
			PartyMemberFrame4Flash:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-PARTYFRAME-FLASH");
			PartyMemberFrame4Flash:SetPoint("TOPLEFT", 0, 6)
		-- Health Bar
			PartyMemberFrame4HealthBar:SetPoint("TOPLEFT",47,-3)
			PartyMemberFrame4HealthBar:SetHeight(17)
		-- Mana Bar
			-- PartyMemberFrame4ManaBar:SetHeight(6)
		-- Background
			PartyMemberFrame4Background:SetPoint("TOPLEFT",46,-3)
			PartyMemberFrame4Background:SetHeight(24)
			PartyMemberFrame4Background:SetWidth(70)
		-- Debuff Frames
			-- PartyMemberFrame4Debuff1:SetPoint("TOPLEFT",135,-12)
			-- PartyMemberFrame4Debuff1:SetWidth(25)
			-- PartyMemberFrame4Debuff1:SetHeight(25)
			-- PartyMemberFrame4Debuff2:SetWidth(25)
			-- PartyMemberFrame4Debuff2:SetHeight(25)
			-- PartyMemberFrame4Debuff3:SetWidth(25)
			-- PartyMemberFrame4Debuff3:SetHeight(25)
			-- PartyMemberFrame4Debuff4:SetWidth(25)
			-- PartyMemberFrame4Debuff4:SetHeight(25)
	end
end


-- Update settings specific to Party Member Unit Frames when in Vehicles
function PartyFramesImproved_PartyMemberFrame_ToVehicleArt(self)
	if not InCombatLockdown() then
		PartyMemberFrame1VehicleTexture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-VEHICLES-PARTYFRAME")
		PartyMemberFrame2VehicleTexture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-VEHICLES-PARTYFRAME")
		PartyMemberFrame3VehicleTexture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-VEHICLES-PARTYFRAME")
		PartyMemberFrame4VehicleTexture:SetTexture("Interface\\Addons\\PartyFramesImproved\\Textures\\PartyFramesImproved-UI-VEHICLES-PARTYFRAME")
	end
end

function PartyFramesImproved_UpdateRange(self)
	if not InCombatLockdown() then
		local id = self:GetID();
		local partyID = "party"..id;
		if UnitIsVisible(partyID) then
			self:SetAlpha(1)
		elseif not UnitIsVisible(partyID) then
			self:SetAlpha(.5)
		end
	end
end

function PartyFramesImproved_SetPartyFrameScale(S)
	if not InCombatLockdown() then 
	-- SCALE UP PARTY MEMBER FRAMES
		PartyMemberFrame1:SetScale(S);
		PartyMemberFrame2:SetScale(S);
		PartyMemberFrame3:SetScale(S);
		PartyMemberFrame4:SetScale(S);
		characterSettings["partyscale"] = S;
	end
end

function PartyFramesImproved_SetPartyFrameY(Y)
	if not InCombatLockdown() then 
	-- SET VERTICAL POSTION OF PARTY MEMBER FRAME
		PartyMemberFrame1:SetPoint("TOPLEFT",CompactRaidFrameManager,"TOPRIGHT",0,Y)
		characterSettings["PartyMemberFrame1Y"] = Y;
	end
end

-- Slashcommand stuff
SLASH_PARTYFRAMESIMPROVED1 = "/partyframesimproved";
SLASH_PARTYFRAMESIMPROVED2 = "/pfi";
SlashCmdList["PARTYFRAMESIMPROVED"] = function(msg, editBox)
	local tokens = tokenize(msg);
	if(table.getn(tokens) > 0 and strlower(tokens[1]) == "reset") then
		StaticPopup_Show("PFI_LAYOUT_RESET");
		PlaySoundFile("Sound\\Creature\\Illidan\\BLACK_Illidan_04.wav")
	elseif(table.getn(tokens) > 0 and strlower(tokens[1]) == "settings") then
		dout(" ----------");
		dout (" ");
		dout ("PartyFramesImproved");
		dout ("Current User Settings:");
		dout (" ");
		dout ("Move = "..characterSettings["PartyMemberFrame1Y"]);
		dout ("Scale = "..(characterSettings["partyscale"]));
		dout (" ");
		dout(" ----------");
	elseif(table.getn(tokens) > 0 and strlower(tokens[1]) == "scale") then
		if(table.getn(tokens) > 1) then
			PartyFramesImproved_SetPartyFrameScale(tokens[2]);
		else
			dout(" ----------");
			dout (" ");
			dout("Please supply a number, between 0.0 and 10.0 as the second parameter.");
			dout("Give 1.2 to 2 a try to get a feel for it.");
			dout (" ");
			dout(" ----------");
		end
	elseif(table.getn(tokens) > 0 and strlower(tokens[1]) == "move") then
		if(table.getn(tokens) > 1) then
			PartyFramesImproved_SetPartyFrameY(tokens[2]);
		else
			dout(" ----------");
			dout (" ");
			dout("Please supply a whole number,")
			dout("I recommend between 150 and -200.0");
			dout("that was NEGATIVE two hundred");
			dout("as the second parameter.");
			dout("");
			dout("The Blizzard location is -20");
			dout("That was NEGATIVE twenty");
			dout (" ");
			dout(" ----------");
		end
	else
		InterfaceOptionsFrame_OpenToCategory(PartyFramesImprovedConfigFrame);
		dout(" ----------");
		dout (" ");
		dout("Too old fashioned for handy sliders?");
		dout("Then step back in time and use these slash commands");
		dout("Valid commands for PartyFramesImproved are:")
		dout("/pfi scale # (scales the party frames)");
		dout("/pfi move # (moves the party frames vertically)");
		dout("/pfi reset   (resets the party frames)");
		dout("/pfi settings   (displayes the current settings)");
		dout (" ");
		dout(" ----------");
	end
end

-- Setup the static popup dialog for slash command to reset the UI to default settings
StaticPopupDialogs["PFI_LAYOUT_RESET"] = {
	text = "Are you sure you want to reset your party frames?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		PartyFramesImproved_ResetSettings();
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true
}

-- Setup the static popup dialog for a UI reset if the Addon finds incomplete saved character settings
StaticPopupDialogs["PFI_LAYOUT_RESETDEFAULT"] = {
	text = "In order for PartyFramesImproved to work properly,\nyour old layout settings had to be reset.\nThis will reload your UI.",
	button1 = "Reset",
	button2 = "Ignore",
	OnAccept = function()
		ReloadUI();
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true
}

-- COLOR HEALTHBARS TO CLASS COLORS
function UnitColor(unit)
	local r, g, b;
	if ( ( not UnitIsConnected(unit) ) or ( UnitIsDeadOrGhost(unit) ) ) then
		--Color it gray
		r, g, b = 0.5, 0.5, 0.5;
	elseif ( UnitIsPlayer(unit) ) then
		--Try to color it by class.
		local localizedClass, englishClass = UnitClass(unit);
		local classColor = RAID_CLASS_COLORS[englishClass];
		if ( classColor ) then
			r, g, b = classColor.r, classColor.g, classColor.b;
		else
			if ( UnitIsFriend("player", unit) ) then
				r, g, b = 0.0, 1.0, 0.0;
			else
				r, g, b = 1.0, 0.0, 0.0;
			end
		end
	else
		r, g, b = UnitSelectionColor(unit);
	end
	
	return r, g, b;
end

-- Bootstrap
function PartyFramesImproved_StartUp(self)
	self:SetScript('OnEvent', function(self, event) self[event](self) end);
	self:RegisterEvent('PLAYER_ENTERING_WORLD');
	self:RegisterEvent('VARIABLES_LOADED');
end

PartyFramesImproved_StartUp(PartyFramesImproved);

-- Table Dump Functions -- http://lua-users.org/wiki/TableSerialization
function print_r (t, indent, done)
  done = done or {}
  indent = indent or ''
  local nextIndent -- Storage for next indentation value
  for key, value in pairs (t) do
    if type (value) == "table" and not done [value] then
      nextIndent = nextIndent or
          (indent .. string.rep(' ',string.len(tostring (key))+2))
          -- Shortcut conditional allocation
      done [value] = true
      print (indent .. "[" .. tostring (key) .. "] => Table {");
      print  (nextIndent .. "{");
      print_r (value, nextIndent .. string.rep(' ',2), done)
      print  (nextIndent .. "}");
    else
      print  (indent .. "[" .. tostring (key) .. "] => " .. tostring (value).."")
    end
  end
end